@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2016 The Qt Company Ltd.
+** Copyright (C) 2017 The Qt Company Ltd.
 ** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the Qt Quick Templates 2 module of the Qt Toolkit.
@@ -52,72 +52,34 @@
 #include <QtQuickTemplates2/private/qquickcontrol_p_p.h>
 #include <QtQuick/private/qquickitemviewtransition_p.h>
 #include <QtQuick/private/qquickitemchangelistener_p.h>
-#include <QtQml/private/qv4persistent_p.h>
+#include <QtQml/private/qv4value_p.h>
+#include <QtCore/qset.h>
+#include <QtCore/qstack.h>
 
 QT_BEGIN_NAMESPACE
 
-class QQmlContext;
-class QQmlComponent;
+class QQmlContextData;
+class QQuickStackElement;
 struct QQuickStackTransition;
 
-class QQuickStackElement : public QQuickItemViewTransitionableItem, public QQuickItemChangeListener
-{
-    QQuickStackElement();
-
-public:
-    ~QQuickStackElement();
-
-    static QQuickStackElement *fromString(const QString &str, QQuickStackView *view);
-    static QQuickStackElement *fromObject(QObject *object, QQuickStackView *view);
-
-    bool load(QQuickStackView *parent);
-    void incubate(QObject *object);
-    void initialize();
-
-    void setIndex(int index);
-    void setView(QQuickStackView *view);
-    void setStatus(QQuickStackView::Status status);
-
-    void transitionNextReposition(QQuickItemViewTransitioner *transitioner, QQuickItemViewTransitioner::TransitionType type, bool asTarget);
-    bool prepareTransition(QQuickItemViewTransitioner *transitioner, const QRectF &viewBounds);
-    void startTransition(QQuickItemViewTransitioner *transitioner, QQuickStackView::Status status);
-
-    void itemDestroyed(QQuickItem *item) override;
-
-    int index;
-    bool init;
-    bool removal;
-    bool ownItem;
-    bool ownComponent;
-    bool widthValid;
-    bool heightValid;
-    QQmlContext *context;
-    QQmlComponent *component;
-    QQuickStackView *view;
-    QPointer<QQuickItem> originalParent;
-    QQuickStackView::Status status;
-    QV4::PersistentValue properties;
-    QV4::PersistentValue qmlCallingContext;
-};
-
 class QQuickStackViewPrivate : public QQuickControlPrivate, public QQuickItemViewTransitionChangeListener
 {
     Q_DECLARE_PUBLIC(QQuickStackView)
 
 public:
-    QQuickStackViewPrivate();
-
     static QQuickStackViewPrivate *get(QQuickStackView *view)
     {
         return view->d_func();
     }
 
-    void setCurrentItem(QQuickItem *item);
+    void warn(const QString &error);
 
-    QList<QQuickStackElement *> parseElements(QQmlV4Function *args, int from = 0);
+    void setCurrentItem(QQuickStackElement *element);
+
+    QList<QQuickStackElement *> parseElements(int from, QQmlV4Function *args, QStringList *errors);
     QQuickStackElement *findElement(QQuickItem *item) const;
     QQuickStackElement *findElement(const QV4::Value &value) const;
-    QQuickStackElement *createElement(const QV4::Value &value);
+    QQuickStackElement *createElement(const QV4::Value &value, QQmlContextData *context, QString *error);
     bool pushElements(const QList<QQuickStackElement *> &elements);
     bool pushElement(QQuickStackElement *element);
     bool popElements(QQuickStackElement *element);
@@ -129,49 +91,33 @@ public:
 
     void viewItemTransitionFinished(QQuickItemViewTransitionableItem *item) override;
     void setBusy(bool busy);
+    void depthChange(int newDepth, int oldDepth);
 
-    bool busy;
-    QVariant initialItem;
-    QQuickItem *currentItem;
-    QList<QQuickStackElement*> removals;
+    bool busy = false;
+    bool removingElements = false;
+    QString operation;
+    QJSValue initialItem;
+    QQuickItem *currentItem = nullptr;
+    QSet<QQuickStackElement*> removing;
+    QList<QQuickStackElement*> removed;
     QStack<QQuickStackElement *> elements;
-    QQuickItemViewTransitioner *transitioner;
+    QQuickItemViewTransitioner *transitioner = nullptr;
 };
 
-struct QQuickStackTransition
+class QQuickStackViewAttachedPrivate : public QObjectPrivate, public QQuickItemChangeListener
 {
-    static QQuickStackTransition popExit(QQuickStackView::Operation operation, QQuickStackElement *element, QQuickStackView *view);
-    static QQuickStackTransition popEnter(QQuickStackView::Operation operation, QQuickStackElement *element, QQuickStackView *view);
-
-    static QQuickStackTransition pushExit(QQuickStackView::Operation operation, QQuickStackElement *element, QQuickStackView *view);
-    static QQuickStackTransition pushEnter(QQuickStackView::Operation operation, QQuickStackElement *element, QQuickStackView *view);
-
-    static QQuickStackTransition replaceExit(QQuickStackView::Operation operation, QQuickStackElement *element, QQuickStackView *view);
-    static QQuickStackTransition replaceEnter(QQuickStackView::Operation operation, QQuickStackElement *element, QQuickStackView *view);
-
-    bool target;
-    QQuickStackView::Status status;
-    QQuickItemViewTransitioner::TransitionType type;
-    QRectF viewBounds;
-    QQuickStackElement *element;
-    QQuickTransition *transition;
-};
-
-class QQuickStackAttachedPrivate : public QObjectPrivate, public QQuickItemChangeListener
-{
-    Q_DECLARE_PUBLIC(QQuickStackAttached)
+    Q_DECLARE_PUBLIC(QQuickStackViewAttached)
 
 public:
-    QQuickStackAttachedPrivate() : element(nullptr) { }
-
-    static QQuickStackAttachedPrivate *get(QQuickStackAttached *attached)
+    static QQuickStackViewAttachedPrivate *get(QQuickStackViewAttached *attached)
     {
         return attached->d_func();
     }
 
     void itemParentChanged(QQuickItem *item, QQuickItem *parent);
 
-    QQuickStackElement *element;
+    bool explicitVisible = false;
+    QQuickStackElement *element = nullptr;
 };
 
 QT_END_NAMESPACE
